@@ -4,6 +4,7 @@ import sys
 import uuid
 
 import pandas as pd
+from google.api_core import protobuf_helpers
 from google.ads.googleads.client import GoogleAdsClient
 from google.ads.googleads.errors import GoogleAdsException
 
@@ -616,6 +617,108 @@ def create_ad(client, customer_id, ad_group_id, final_url, headlines, descriptio
             f"Created responsive search ad with resource name "
             f'"{result.resource_name}".'
         )
+    
+def update_adgroup(client,customer_id, ad_group_id, **kwargs):
+    if len(kwargs) > 0:
+        ad_group_service = client.get_service("AdGroupService")
+
+        # Create ad group operation.
+        ad_group_operation = client.get_type("AdGroupOperation")
+        ad_group = ad_group_operation.update
+        ad_group.resource_name = ad_group_service.ad_group_path(
+            customer_id, ad_group_id
+        )
+        if 'status' in kwargs:
+            stat = kwargs['status']
+            if stat == 'PAUSED':
+                ad_group.status = client.enums.AdGroupStatusEnum.PAUSED
+            elif stat == 'ENABLED':
+                ad_group.status = client.enums.AdGroupStatusEnum.ENABLED
+            elif stat == 'UNSPECIFIED':
+                ad_group.status = client.enums.AdGroupStatusEnum.UNSPECIFIED
+        if 'cpc_bid_micro_amount' in kwargs:
+            ad_group.cpc_bid_micros = kwargs['cpc_bid_micro_amount']
+        client.copy_from(
+            ad_group_operation.update_mask,
+            protobuf_helpers.field_mask(None, ad_group._pb),
+        )
+        # Update the ad group.
+        ad_group_response = ad_group_service.mutate_ad_groups(
+            customer_id=customer_id, operations=[ad_group_operation]
+        )
+
+        print(f"Updated ad group {ad_group_response.results[0].resource_name}.")
+    else:
+        print('nothing to update')
+        
+
+def update_campaign(client,customer_id, campaign_id, **kwargs):
+    if len(kwargs) > 0:
+        campaign_service = client.get_service("CampaignService")
+
+        # Create ad group operation.
+        campaign_operation = client.get_type("CampaignOperation")
+        campaign = campaign_operation.update
+        campaign.resource_name = campaign_service.campaign_path(
+            customer_id, campaign_id
+        )
+        if 'status' in kwargs:
+            stat = kwargs['status']
+            if stat == 'PAUSED':
+                campaign.status = client.enums.CampaignStatusEnum.PAUSED
+            elif stat == 'ENABLED':
+                campaign.status = client.enums.CampaignStatusEnum.ENABLED
+            elif stat == 'UNSPECIFIED':
+                campaign.status = client.enums.CampaignStatusEnum.UNSPECIFIED
+        if 'cpc_bid_micro_amount' in kwargs:
+            campaign.cpc_bid_micros = kwargs['cpc_bid_micro_amount']
+        client.copy_from(
+            campaign_operation.update_mask,
+            protobuf_helpers.field_mask(None, campaign._pb),
+        )
+        # Update the ad group.
+        campaign_response = campaign_service.mutate_campaigns(
+            customer_id=customer_id, operations=[campaign_operation]
+        )
+
+        print(
+            f"Updated campaign {campaign_response.results[0].resource_name}.")
+    else:
+        print('nothing to update')
+        
+
+def update_keyword(client,customer_id, ad_group_id, criterion_id, **kwargs):
+    if len(kwargs) > 0:
+        agc_service = client.get_service("AdGroupCriterionService")
+        ad_group_criterion_operation = client.get_type("AdGroupCriterionOperation")
+
+        ad_group_criterion = ad_group_criterion_operation.update
+        ad_group_criterion.resource_name = agc_service.ad_group_criterion_path(
+            customer_id, ad_group_id, criterion_id
+        )
+        if 'status' in kwargs:
+            stat = kwargs['status']
+            if stat == 'PAUSED':
+                ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.PAUSED
+            elif stat == 'ENABLED':
+                ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.ENABLED
+            elif stat == 'UNSPECIFIED':
+                ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.UNSPECIFIED
+        if 'cpc_bid_micro_amount' in kwargs:
+            ad_group_criterion.cpc_bid_micros = kwargs['cpc_bid_micro_amount']
+        # ad_group_criterion.final_urls.append("https://www.example.com")
+        client.copy_from(
+            ad_group_criterion_operation.update_mask,
+            protobuf_helpers.field_mask(None, ad_group_criterion._pb),
+        )
+
+        agc_response = agc_service.mutate_ad_group_criteria(
+            customer_id=customer_id, operations=[ad_group_criterion_operation]
+        )
+        print(f"Updated keyword {agc_response.results[0].resource_name}.")
+    else:
+        print('nothing to update')
+
 
 def remove_campaign(client, customer_id, campaign_id):
     campaign_service = client.get_service("CampaignService")
@@ -771,6 +874,100 @@ def get_search_term_report(client, customer_id,start_date='2022-01-01', end_date
     df = pd.DataFrame.from_dict(df_dict)
     return df
 
+df_dict = {
+    'date' : list(),
+    'camp_id' : list(),
+    'camp_name' : list(),
+    'camp_status' : list(),
+    'adgroup_id' : list(),
+    'adgroup_name' : list(),
+    'adgroup_status' : list(),
+    'ad_group_criterion_resource_name' : list(),
+    'adgroup_criterion_id' : list(),
+    'ad_group_criterion_keyword_text' : list(),
+    'ad_group_criterion_keyword_match_type' : list(),
+    'adgroup_criterion_status' : list(),
+    'metrics_impressions' : list(),
+    'metrics_clicks' : list(),
+    'metrics_cost' : list(),
+    'metrics_cpc' : list(),
+    'metrics_cost_per_conversion' : list(),
+    'metrics_cost_per_all_conversions' : list(),
+    'metrics_all_conversions_by_conversion_date' : list(),
+    'metrics_all_conversions' : list(),
+    }
+
+
+def get_keyword_stats(client, customer_id, start_date = '2020-09-01', end_date = '2022-02-25'):
+    ga_service = client.get_service("GoogleAdsService")
+    start = datetime.datetime.strptime(start_date, '%Y-%m-%d')
+    end = datetime.datetime.strptime(end_date, '%Y-%m-%d')
+    step = datetime.timedelta(days=1)
+    while start <= end:
+        query1 = """
+            SELECT
+            campaign.id,
+            campaign.name,
+            campaign.status,
+            ad_group.id,
+            ad_group.name,
+            ad_group.status,
+            ad_group_criterion.resource_name,
+            ad_group_criterion.criterion_id,
+            ad_group_criterion.keyword.text,
+            ad_group_criterion.keyword.match_type,
+            ad_group_criterion.status,
+            metrics.impressions,
+            metrics.clicks,
+            metrics.cost_micros,
+            metrics.average_cpc,
+            metrics.cost_per_conversion, 
+            metrics.cost_per_all_conversions,
+            metrics.all_conversions_by_conversion_date, 
+            metrics.all_conversions"""
+        query2 = f"FROM keyword_view WHERE segments.date >= '{str(start)[0:10]}' AND segments.date < '{str(start+step)[0:10]}'"
+        query3 = """AND campaign.advertising_channel_type = 'SEARCH'
+            AND ad_group.status = 'ENABLED'
+            AND ad_group_criterion.status IN ('ENABLED', 'PAUSED')
+            ORDER BY metrics.impressions DESC
+            """
+        fin_query = query1 + '\n' + query2 + '\n' + query3
+        
+        search_request = client.get_type("SearchGoogleAdsStreamRequest")
+        search_request.customer_id = customer_id
+        search_request.query = fin_query
+        stream = ga_service.search_stream(search_request)
+        for batch in stream:
+            for row in batch.results:
+                campaign = row.campaign
+                ad_group = row.ad_group
+                criterion = row.ad_group_criterion
+                metrics = row.metrics
+                df_dict['date'].append(start)
+                df_dict['camp_id'].append(campaign.id)
+                df_dict['camp_name'].append(campaign.name)
+                df_dict['camp_status'].append(campaign.status.name)
+                df_dict['adgroup_id'].append(ad_group.id)
+                df_dict['adgroup_name'].append(ad_group.name)
+                df_dict['adgroup_status'].append(ad_group.status.name)
+                df_dict['ad_group_criterion_resource_name'].append(criterion.resource_name)
+                df_dict['adgroup_criterion_id'].append(criterion.criterion_id)
+                df_dict['ad_group_criterion_keyword_text'].append(criterion.keyword.text)
+                df_dict['ad_group_criterion_keyword_match_type'].append(criterion.keyword.match_type.name)
+                df_dict['adgroup_criterion_status'].append(criterion.status.name)
+                df_dict['metrics_impressions'].append(metrics.impressions)
+                df_dict['metrics_clicks'].append(metrics.clicks)
+                df_dict['metrics_cost'].append(metrics.cost_micros)
+                df_dict['metrics_cpc'].append(metrics.average_cpc)
+                df_dict['metrics_cost_per_conversion'].append(metrics.cost_per_conversion)
+                df_dict['metrics_cost_per_all_conversions'].append(metrics.cost_per_all_conversions)
+                df_dict['metrics_all_conversions'].append(metrics.all_conversions)
+                df_dict['metrics_all_conversions_by_conversion_date'].append(
+                    metrics.all_conversions_by_conversion_date)
+        start = start+step
+    df = pd.DataFrame.from_dict(df_dict)
+    return df
+
 def _handle_googleads_exception(exception):
     print(
         f'Request with ID "{exception.request_id}" failed with status '
