@@ -1,10 +1,11 @@
 import os
+import re
 import uuid
 import numpy as np
 import pandas as pd
 from google.ads.googleads.client import GoogleAdsClient
 from sentence_transformers import SentenceTransformer
-from autoads.models import get_similarity_matrix
+from autoads.models import get_similarity_matrix,get_similarity_api
 from autoads.keywords import get_keywords_metrics,make_live_request,get_results
 from autoads.gads import get_existing_keywords
 
@@ -17,6 +18,7 @@ path = 'data/google-ads.yaml'
 customer_id = '8306215642' #google ads customer id
 first_threshold = 0.8 # similarity threshold for keyword similarity
 second_threshold = 0.8 # similarity threshold for description similarity
+third_threshold = 0.35 # similarity threshold for seed keyword and keywords3
 conversion_threshold = 1 #  get conversion >= conversion_threshold
 allowed_funnels = ['LF','MF'] # available options LF, MF, UF
 similarity_model_path = 'Maunish/ecomm-sbert' # this model is already uploaded on huggingface so no need to download
@@ -28,8 +30,10 @@ df = pd.read_csv(df_path)
 
 print("Getting existing keywords")
 df_existing = get_existing_keywords(googleads_client,customer_id)
-# df_existing = pd.read_csv(save_path+'/df_existing.csv')
-keywords_to_remove = df_existing[df_existing['camp_status'].isin(['ENABLED'])]['keyword_name'].unique().tolist()
+df_existing = pd.read_csv(save_path+'/df_existing.csv')
+keywords_to_remove = df_existing.query("(camp_status == 'ENABLED') & \
+                                        (adgroup_status == 'ENABLED') &\
+                                        (keyword_status == 'ENABLED')")['keyword_name'].unique().tolist()
 df = df[~df["Keywords"].isin(keywords_to_remove)] # if campaign in enabled and keyword clash remove it.
 
 #First pass of filters
@@ -53,10 +57,10 @@ print("Getting meta titles from SERP")
 seed_keywords = df["Keywords2"].unique().tolist()
 for keyword in seed_keywords:
     print(keyword)
-    response,_ = make_live_request(keyword,email,api_key)
+    response = make_live_request(keyword,email,api_key,depth=5)
     results = get_results(response,tasks,task_params)
     if results.get('organic_description') and response['tasks'][0]['result']:
-        organic_description = str(results['organic_description'])
+        organic_description = str([x.replace(",","") for x in results['organic_description']])
         organic_description = organic_description.replace("[","").replace("]","")
         df.loc[df["Keywords2"]== keyword,'seed_organic_description'] = str(organic_description)
     else:
@@ -65,14 +69,14 @@ for keyword in seed_keywords:
 all_keywords = df["Keywords"].unique().tolist()
 for keyword in all_keywords:
     print(keyword)
-    response,_ = make_live_request(keyword,email,api_key)
+    response = make_live_request(keyword,email,api_key,depth=5)
     results = get_results(response,tasks,task_params)
     if results.get('organic_description') and response['tasks'][0]['result']:
-        organic_description = str(results['organic_description'])
+        organic_description = str([x.replace(",","") for x in results['organic_description']])
         organic_description = organic_description.replace("[","").replace("]","")
         df.loc[df["Keywords"]== keyword,'organic_description'] = organic_description
     else:
-         df.loc[df["Keywords"]== keyword,'organic_description'] = [keyword]  
+        df.loc[df["Keywords"]== keyword,'organic_description'] = [keyword]  
 
 model = SentenceTransformer(similarity_model_path)
 
@@ -85,8 +89,26 @@ for i,row in df.iterrows():
 
 df = df[df['second_similarity'] >= second_threshold]
 
-df_enabled = df_existing[df_existing['camp_status'] == 'ENABLED']
-df_paused = df_existing[df_existing['camp_status'] == 'PAUSED']
+def remove_word(row):
+    keyword1 = re.split("\s|(?<!\d)[,-](?!\d)",row['Keywords'])
+    keyword2 = re.split("\s|(?<!\d)[,-](?!\d)",row['Keywords2'])
+    keyword = [x for x in keyword1 if x not in keyword2]
+    return "" if len(keyword) == 0 else ' '.join(keyword)
+
+df['Keywords3'] = df.apply(remove_word,axis=1)
+
+keywords1 = df['Keywords2'].tolist()
+keywords2 = df['Keywords3'].tolist()
+df['third_similarity'] = get_similarity_api(model,keywords1,keywords2)
+
+df = df[df['third_similarity'] >= third_threshold]
+
+df_enabled = df_existing.query("(camp_status == 'ENABLED') & \
+                                (adgroup_status == 'ENABLED') &\
+                                (keyword_status == 'ENABLED') & \
+                                (camp_experiment_type != 'EXPERIMENT')")
+
+df_paused = df_existing.query("camp_status == 'PAUSED'")
 
 if len(df) != 0:
     if len(df_enabled) == 0:
