@@ -32,6 +32,7 @@ keywords_to_remove = df_existing[df_existing['camp_status'].isin(['ENABLED'])]['
 print("Getting search term report")
 df_search_term_report = get_search_term_report(googleads_client,customer_id,start_date,end_date)
 # df_search_term_report = pd.read_csv('data/search_term_df.csv')
+df_search_term_report = df_search_term_report[~df_search_term_report['stv_status'].isin(['EXCLUDED','ADDED_EXCLUDED'])]
 df_search_term_report = df_search_term_report[df_search_term_report['campaign_status'].isin(['ENABLED'])]
 df_search_term_report['sum_conversion'] = df_search_term_report.groupby('date')['metrics_conversions'].transform(sum)
 df_search_term_report_negative = df_search_term_report.loc[df_search_term_report['sum_conversion']==0]
@@ -180,17 +181,16 @@ if len(df_search_term_report) != 0:
 else:
     print("No keywords to add into existing campaigns")
 
-# path = '/home/maunish/Upwork Projects/Google-Ads-Project/examples/google-ads.yaml'
-# customer_id = '6554081276' # google ads customer id
-# googleads_client = GoogleAdsClient.load_from_storage(path=path, version="v9")
 
 if len(df_search_term_report_negative) != 0:
     print("Calculating similarity")
     model = SentenceTransformer(similarity_model_path)
+    df_search_term_report_negative = df_search_term_report_negative[~df_search_term_report_negative['stv_status'].isin(['EXCLUDED','ADDED_EXCLUDED'])]
     keywords1 = df_search_term_report_negative['stv_search_term'].tolist()
     keywords2 = df_search_term_report_negative['keyword_name'].tolist()
     df_search_term_report_negative['similarity'] = get_similarity_api(model,keywords1,keywords2)    
     df_search_term_report_negative = df_search_term_report_negative[df_search_term_report_negative['similarity']<=negative_threshold]
+    df_search_term_report_negative = df_search_term_report_negative[~df_search_term_report_negative['stv_status'].isin(['EXCLUDED','ADDED_EXCLUDED'])]
     df_search_term_report_negative.to_csv(save_path+'/df_search_term_report_negative.csv',index=False)
     if len(df_search_term_report_negative) != 0:
         answer = input("Do you want to upload negative keywords? (y/n): ")
